#include "xmameconfig.h"

XMameCfg::XMameCfg(QWidget *parent) : KDialogBase(IconList, i18n("Preferences"), Ok|Cancel, Ok, parent, 0, false)
{
//general connections...
	connect(&adProc, SIGNAL( receivedStdout(KProcess *, char *, int)), this, SLOT( setDefaultPaths( KProcess *, char *, int)) );
	connect(&nfoProc, SIGNAL( receivedStdout(KProcess *, char *, int)), this, SLOT( setVersionInfo( KProcess *, char *, int)) );

	config = kapp->config();
	resize(400,300);
	loadModules();
}


XMameCfg::~XMameCfg()
{
}

void XMameCfg::loadModules()
{
//here is the list of loaded modules...
	modGeneral();
	modPaths();
	modScreen();
	modAudio();
	modVideo();
	modEffects();
	modDevices();
	modInfo();

//we won't need the module for first time more than JUST a first time...
	config->setGroup("First_run");
	if(config->readBoolEntry("First",true) == true)
		modFirstRun();
	else
		loadConfig();
}

void XMameCfg::modInfo()
{
	QFrame *page1 = addPage( i18n("Info"), i18n("Information"), locate("data","kmameleon/pics/mod_info.png") );

	QLabel* textLabel1;
	QGridLayout* Form1Layout;
	QGridLayout* layout5;

	Form1Layout = new QGridLayout( page1, 1, 1, 5, 5, "Form1Layout");

	layout5 = new QGridLayout( 0, 1, 1, 0, 4, "layout5");

	versionBrowser = new QTextBrowser( page1, "versionBrowser" );
	versionBrowser->setTextFormat( QTextBrowser::PlainText );

	layout5->addWidget( versionBrowser, 1, 0 );
	textLabel1 = new QLabel( page1, "textLabel1" );
	textLabel1 -> setText (i18n("XMame version information:"));
	layout5->addWidget( textLabel1, 0, 0 );
	Form1Layout->addLayout( layout5, 0, 0 );


	QStringList xMameBinList;
	//this will be overwritten unconditinally

	pthXmameSdlBin->setText(KStandardDirs::findExe("xmame.SDL"));
	pthXmameX11Bin->setText(KStandardDirs::findExe("xmame.x11"));
	pthXmameGlBin->setText(KStandardDirs::findExe("xmame.xgl"));
	pthXmameSvgaLibBin->setText(KStandardDirs::findExe("xmame.svgalib"));

//could this be easier?
	if (KStandardDirs::findExe("xmame.SDL") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.SDL");
	if (KStandardDirs::findExe("xmame.x11") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.x11");
	if (KStandardDirs::findExe("xmame.xgl") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.xgl");
	if (KStandardDirs::findExe("xmame.svgalib") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.svgalib");

	versionBrowser -> insert(i18n("Executables found: %1\n").arg(xMameBinList.count()));
	versionBrowser -> insert(i18n("Locations:\n"));
	for (unsigned int xx = 0; xx < xMameBinList.size(); xx++)
	{
		if (! xMameBinList[xx].isEmpty())
			versionBrowser -> insert(xMameBinList[xx] + "\n");
	}

	kdDebug() << "Executables count: " << xMameBinList.count() << endl;

	if (xMameBinList.count() == 1)
	{
		QString options = xMameBinList[0];
		options +=" --version ";

		nfoProc.clearArguments();
		#if KDE_VERSION >= 306
			nfoProc.setUseShell(true);
		#endif

		nfoProc << options.local8Bit();
		if (nfoProc.start(KProcess::NotifyOnExit, KProcess::AllOutput) == false)
			kdDebug() << "something wrong" << endl;
	}
}

void XMameCfg::modGeneral()
{
	QFrame *page1 = addPage( i18n("General"), i18n("General"), locate("data","kmameleon/pics/mod_general.png") );
	QVBoxLayout *topLayout = new QVBoxLayout( page1, 0, 6 );

	QLabel *label = new QLabel( i18n("XMame Type:"), page1 );
	QLabel *label2 = new QLabel( i18n("Configuration option:"), page1 );
	xMameType = new QComboBox(page1);
	chkOutput = new QCheckBox(i18n("Show output"), page1);

	chkColorOutput = new QCheckBox(i18n("Show colorful output"), page1);
	chkCheatMode = new QCheckBox(i18n("Enable cheat mode (Needs a cheat file)"), page1);
	chkDetachProcess = new QCheckBox(i18n("Detach xmame process (it will close xmame process on kmameleon exit if left unchecked)"), page1);

//adding the XMame types to the list...
	xMameType->insertItem( i18n( "[autodetect]" ) );
	xMameType->insertItem( i18n( "SDL" ) );
	xMameType->insertItem( i18n( "X11" ) );
	xMameType->insertItem( i18n( "OpenGL" ) );
	xMameType->insertItem( i18n( "SVGALIB" ) );

	cbConfigType = new QComboBox(page1);
	cbConfigType->insertItem(i18n("Simple"));
	cbConfigType->insertItem(i18n("Advanced"));

	QLabel *lblInfo = new QLabel( i18n("Simple option lets You configure most of the basic things, that You need...\nAdvanced option will let You enable all the options... This is for advanced users only"), page1);

	topLayout->addWidget( label,0,0 );
	topLayout->addWidget( xMameType,1,0 );

	topLayout->addWidget( chkOutput,2,0 );
	topLayout->addWidget( chkColorOutput,3,0 );
	topLayout->addWidget( chkCheatMode,4,0 );
	topLayout->addWidget( chkDetachProcess,5,0 );
	topLayout->addWidget( label2,6,0 );
	topLayout->addWidget( cbConfigType,7,0 );
	topLayout->addWidget( lblInfo,8,0 );

	connect (cbConfigType, SIGNAL(activated(int)), this, SLOT(slotConfigType(int)) );

	QSpacerItem* spacer = new QSpacerItem( 0, 210, QSizePolicy::Minimum, QSizePolicy::Expanding );
	topLayout->addItem( spacer );
}

void XMameCfg::modAudio()
{
	QFrame *page = addPage( i18n("Audio"), i18n("Audio"), locate("data","kmameleon/pics/mod_audio.png") );
	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, 6 );

	chkIsSound = new QCheckBox(i18n("Enable XMame sound"), page);
	chkSamples = new QCheckBox(i18n("Enable samples (if avaliable)"), page);
	chkFakeSound = new QCheckBox(i18n("Enable fake sound (some games may not run without sound)"), page);

	topLayout->addWidget( chkIsSound,0,0 );
	topLayout->addWidget( chkSamples,1,0 );
	topLayout->addWidget( chkFakeSound,2,0 );

	QGroupBox *audioBox = new QGroupBox(page, "audioBox");
	audioBox->setTitle("Advanced audio options");
	QGridLayout* audioBoxLayout;

//Volume control
	audioBox->setColumnLayout(0, Qt::Vertical );
	audioBox->layout()->setSpacing( 6 );
	audioBox->layout()->setMargin( 11 );
	audioBoxLayout = new QGridLayout( audioBox->layout() );
	audioBoxLayout->setAlignment( Qt::AlignTop );

	audioSetVolume = new QSlider( audioBox, "audioSetVolume" );
	audioSetVolume->setMinValue( -32 );
	audioSetVolume->setMaxValue( 0 );
	audioSetVolume->setOrientation( QSlider::Horizontal );
	audioSetVolume->setTickmarks( QSlider::Left );
	audioSetVolume->setTickInterval( 5 );
	audioSetVolume->setValue( -32 );

	connect(audioSetVolume, SIGNAL(valueChanged(int)), this, SLOT(changeVolumeLevel(int)) );

	audioBoxLayout->addMultiCellWidget( audioSetVolume, 1, 1, 0, 8 );

	QLabel *lblVolume = new QLabel( audioBox, "lblVolume" );
	lblVolume->setText(i18n("Volume:"));
	audioBoxLayout->addMultiCellWidget( lblVolume, 0, 0, 0, 8 );

	QLabel *lblSofter = new QLabel( audioBox, "lblSofter" );
	lblSofter->setText(i18n("Softer"));
	audioBoxLayout->addWidget( lblSofter, 2, 0 );

	QLabel *lblLouder = new QLabel( audioBox, "lblLouder" );
	lblLouder->setText(i18n("Louder"));
	audioBoxLayout->addWidget( lblLouder, 2, 8 );

	QSpacerItem* spacer_2 = new QSpacerItem( 60, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioBoxLayout->addItem( spacer_2, 2, 1 );
	QSpacerItem* spacer_3 = new QSpacerItem( 80, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioBoxLayout->addItem( spacer_3, 2, 3 );

	lblDisplayVolume = new QLabel( audioBox, "lblDisplayVolume" );
	lblDisplayVolume->setText(i18n("-32 dB"));
	audioBoxLayout->addMultiCellWidget( lblDisplayVolume, 2, 2, 2, 3);

//everything else :)
	QFrame *line1 = new QFrame( audioBox, "line1" );
	line1->setFrameShape( QFrame::HLine );
	line1->setFrameShadow( QFrame::Sunken );
	line1->setFrameShape( QFrame::HLine );
	audioBoxLayout->addMultiCellWidget( line1, 3, 3, 0, 8 );

	QLabel *lblSound = new QLabel( audioBox, "lblSound" );
	lblSound->setText(i18n("Sound options"));
	audioBoxLayout->addMultiCellWidget( lblSound, 4, 4, 0, 1 );

	chkCustomAudioSf = new QCheckBox( audioBox, "chkCustomAudioSf" );
	chkCustomAudioSf->setText(i18n("Custom sample frequency"));
	audioBoxLayout->addMultiCellWidget( chkCustomAudioSf, 5, 5, 0, 3 );

	QLabel *lblDevices = new QLabel( audioBox, "lblDevices" );
	lblDevices->setText(i18n("Devices"));
	audioBoxLayout->addWidget( lblDevices, 4, 5 );


	chkCustomAudioMixerDevice = new QCheckBox( audioBox, "chkCustomAudioMixerDevice" );
	chkCustomAudioMixerDevice->setText(i18n("Custom mixer device"));
	audioBoxLayout->addMultiCellWidget( chkCustomAudioMixerDevice, 5, 5, 5, 8 );
	QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	audioBoxLayout->addItem( spacer_4, 7, 1 );

	chkCustomAudioBufSize = new QCheckBox( audioBox, "chkCustomAudioBufSize" );
	chkCustomAudioBufSize->setText(i18n("Custom sound frames to buffer"));
	audioBoxLayout->addMultiCellWidget( chkCustomAudioBufSize, 8, 8, 0, 3 );

	chkCustomAudioDevice = new QCheckBox( audioBox, "chkCustomAudioDevice" );
	chkCustomAudioDevice->setText(i18n("Custom audio device"));
	audioBoxLayout->addMultiCellWidget( chkCustomAudioDevice, 8, 8, 5, 8 );
	QSpacerItem* spacer_5 = new QSpacerItem( 20, 16, QSizePolicy::Minimum, QSizePolicy::Expanding );
	audioBoxLayout->addItem( spacer_5, 7, 6 );


	audioBufSizeEdit = new QLineEdit( audioBox, "audioBufSizeEdit" );
	audioBoxLayout->addMultiCellWidget( audioBufSizeEdit, 9, 9, 0, 2 );

	audioSfEdit = new QLineEdit( audioBox, "audioSfEdit" );
	audioBoxLayout->addMultiCellWidget( audioSfEdit, 6, 6, 0, 1 );

	QSpacerItem* spacer_6 = new QSpacerItem( 21, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioBoxLayout->addItem( spacer_6, 9, 3 );
	QSpacerItem* spacer_7 = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioBoxLayout->addItem( spacer_7, 6, 3 );

	audioMixerDeviceEdit = new QLineEdit( audioBox, "audioMixerDeviceEdit" );
	audioBoxLayout->addMultiCellWidget( audioMixerDeviceEdit, 6, 6, 5, 7 );

	audioDeviceEdit = new QLineEdit( audioBox, "audioDeviceEdit" );
	audioBoxLayout->addMultiCellWidget( audioDeviceEdit, 9, 9, 5, 7 );

	QSpacerItem* spacer_8 = new QSpacerItem( 21, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioBoxLayout->addItem( spacer_8, 6, 8 );
	QSpacerItem* spacer_9 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioBoxLayout->addItem( spacer_9, 9, 8 );

	topLayout->addWidget(audioBox,4,0);

//this  is on the very end... To keep "things up" :)
	QSpacerItem* spacer = new QSpacerItem( 0, 210, QSizePolicy::Minimum, QSizePolicy::Expanding );
	topLayout->addItem( spacer );
}

void XMameCfg::modVideo()
{
	QFrame *page = addPage( i18n("Video"), i18n("Video"), locate("data","kmameleon/pics/mod_video.png") );
	QVBoxLayout *topLayout = new QVBoxLayout( page, 0, 6 );
	QHBoxLayout *layer_mf = new QHBoxLayout (page, 0, 0,"layer_mf");
	QHBoxLayout *layer_mf2 = new QHBoxLayout (page, 0, 0,"layer_mf2");

	QLabel *lblVideoFx = new QLabel(i18n("Video effect:"), page);
	videoFx = new QComboBox(page);

	videoFx -> insertItem(i18n("none (default)"));
	videoFx -> insertItem(i18n("scale 2x (smooth scaling effect)"));
	videoFx -> insertItem(i18n("scan 2 (light scanlines)"));
	videoFx -> insertItem(i18n("rgbstripe (3x2 rgb vertical stripes)"));
	videoFx -> insertItem(i18n("rgbscan (2x3 rgb horizontal scanlines)"));
	videoFx -> insertItem(i18n("scan3 (3x3 deluxe scanlines)"));

	topLayout->addWidget(lblVideoFx, 0, 0);
	topLayout->addWidget(videoFx, 1, 0);

	QLabel *lblBpp = new QLabel(i18n("Specify the colordepth the core should render:"), page);
	cbBpp = new QComboBox(page);
	cbBpp -> insertItem(i18n("Auto"));
	cbBpp -> insertItem(i18n("8"));
	cbBpp -> insertItem(i18n("16"));

	chkEnableFrameSkip = new QCheckBox(i18n("Enable frame skipping"), page);

	chkFrameSkip = new QCheckBox(i18n("Auto frame skipping"), page);
	QLabel *lblMaxAutoFrameSkip = new QLabel (i18n("Max auto frames skipped:"), page);
	spnMaxAutoFrameSkip = new QSpinBox(page);

	QLabel *lblNoAutoFrameSkip = new QLabel (i18n("Frameskip when not autoframeskiping:"), page);
	spnNoAutoFrameSkip = new QSpinBox(page);

//layout will go at the very end...
	topLayout->addWidget(lblBpp, 2, 0);
	topLayout->addWidget(cbBpp, 3, 0);
	topLayout->addWidget(chkEnableFrameSkip, 4, 0);
	topLayout->addWidget(chkFrameSkip, 5, 0);

	layer_mf ->addWidget(lblMaxAutoFrameSkip,0,0);
	QSpacerItem* xSpacer = new QSpacerItem( 111, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layer_mf ->addItem(xSpacer);
	layer_mf ->addWidget(spnMaxAutoFrameSkip,1,0);

	layer_mf2 ->addWidget(lblNoAutoFrameSkip,0,0);
	QSpacerItem* xSpacer2 = new QSpacerItem( 111, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layer_mf2 ->addItem(xSpacer2);
	layer_mf2 ->addWidget(spnNoAutoFrameSkip,1,0);

	QGridLayout* rotationBoxLayout;

	topLayout->addLayout(layer_mf, 6);
	topLayout->addLayout(layer_mf2, 7);

	rotationBox = new QGroupBox( page, "rotationBox" );
	rotationBox->setColumnLayout(0, Qt::Vertical );
	rotationBox->layout()->setSpacing( 4 );
	rotationBox->layout()->setMargin( 4 );
	rotationBoxLayout = new QGridLayout( rotationBox->layout() );
	rotationBoxLayout->setAlignment( Qt::AlignTop );
	rotationBox->setTitle( i18n( "Rotaton" ) );

	QHBoxLayout* layout5;
	layout5 = new QHBoxLayout( 0, 0, 4, "layout5");

	QVBoxLayout* layout26;
	layout26 = new QVBoxLayout( 0, 0, 4, "layout26");

	QVBoxLayout* layout24;
	layout24 = new QVBoxLayout( 0, 0, 4, "layout24");

	chkFlipX = new QCheckBox( rotationBox, "chkFlipX" );
	chkFlipX->setText( i18n( "Flip X Axis" ) );
	layout24->addWidget( chkFlipX );

	chkFlipY = new QCheckBox( rotationBox, "chkFlipY" );
	chkFlipY->setText( i18n( "Flip Y Axis" ) );
	layout24->addWidget( chkFlipY );
	layout26->addLayout( layout24 );
	QSpacerItem* spacer1 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layout26->addItem( spacer1 );
	layout5->addLayout( layout26 );
	QSpacerItem* spacer2 = new QSpacerItem( 140, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout5->addItem( spacer2 );

	QVBoxLayout* layout25;
	layout25 = new QVBoxLayout( 0, 0, 4, "layout25");

	cbRotation = new QComboBox( FALSE, rotationBox, "cbRotation" );
	layout25->addWidget( cbRotation );
	QSpacerItem* spacer3 = new QSpacerItem( 20, 21, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layout25->addItem( spacer3 );
	layout5->addLayout( layout25 );

	rotationBoxLayout->addLayout( layout5, 0, 0 );

	cbRotation->clear();
	cbRotation->insertItem( i18n( "Disable rotation" ) );
	cbRotation->insertItem( i18n( "No rotation (not the same as disable)" ) );
	cbRotation->insertItem( i18n( "Rotate 90 degrees left" ) );
	cbRotation->insertItem( i18n( "Rotate 90 degrees right" ) );
	topLayout->addWidget(rotationBox, 1, 0);

	QSpacerItem* spacer = new QSpacerItem( 0, 210, QSizePolicy::Minimum, QSizePolicy::Expanding );
	topLayout->addItem( spacer );
}

void XMameCfg::modEffects()
{
	QFrame *page7 = addPage( i18n("Video FX"), i18n("Video Effects"), locate("data","kmameleon/pics/mod_fx.png") );

	QGridLayout* Form1Layout;
	Form1Layout = new QGridLayout( page7, 1, 1, 11, 6, "Form1Layout");

	QHBoxLayout* layout29;
	QVBoxLayout* layout22;
	layout29 = new QHBoxLayout( 0, 0, 6, "layout29");

	layout22 = new QVBoxLayout( 0, 0, 6, "layout22");

	chkScanlines = new QCheckBox( page7, "chkScanlines" );
	chkScanlines->setText( i18n( "Enable/disable displaying\nsimulated scanlines" ) );
	layout22->addWidget( chkScanlines );

	chkArtwork = new QCheckBox( page7, "chkArtwork" );
	chkArtwork->setText( i18n( "Use additional game artwork" ) );
	layout22->addWidget( chkArtwork );

	chkBackdrops = new QCheckBox( page7, "chkBackdrops" );
	chkBackdrops->setText( i18n( "Use backdrop artwork" ) );
	layout22->addWidget( chkBackdrops );

	chkOverlay = new QCheckBox( page7, "chkOverlay" );
	chkOverlay->setText( i18n( "Use overlay artwork" ) );
	layout22->addWidget( chkOverlay );

	chkBezel = new QCheckBox( page7, "chkBezel" );
	chkBezel->setText( i18n( "Use bezel artwork" ) );
	layout22->addWidget( chkBezel );

	chkArtCrop = new QCheckBox( page7, "chkArtCrop" );
	chkArtCrop->setText( i18n( "Crop artwork to game screen only" ) );
	layout22->addWidget( chkArtCrop );

	chkThrottle = new QCheckBox( page7, "chkThrottle" );
	chkThrottle->setText( i18n( "Enable/disable throttle" ) );
	layout22->addWidget( chkThrottle );

	chkSleepIdle = new QCheckBox( page7, "chkSleepIdle" );
	chkSleepIdle->setText( i18n( "Enable/disable sleep during idle" ) );
	layout22->addWidget( chkSleepIdle );
	layout29->addLayout( layout22 );
	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout29->addItem( spacer );

	QVBoxLayout* layout28;
	QVBoxLayout* layout21;
	layout28 = new QVBoxLayout( 0, 0, 6, "layout28");

	layout21 = new QVBoxLayout( 0, 0, 6, "layout21");

	QLabel* textLabel7;
	textLabel7 = new QLabel( page7, "textLabel7" );
	textLabel7->setText( i18n( "Set the brightness (0-100%)" ) );
	textLabel7->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	layout21->addWidget( textLabel7 );

	QHBoxLayout* layout20;
	layout20 = new QHBoxLayout( 0, 0, 6, "layout20");
	QSpacerItem* spacer_2 = new QSpacerItem( 21, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout20->addItem( spacer_2 );

	spnBrightness = new QSpinBox( page7, "spnBrightness" );
	layout20->addWidget( spnBrightness );
	layout21->addLayout( layout20 );
	layout28->addLayout( layout21 );

	QVBoxLayout* layout23;
	layout23 = new QVBoxLayout( 0, 0, 6, "layout23");

	QLabel* textLabel6;
	textLabel6 = new QLabel( page7, "textLabel6" );
	textLabel6->setText( i18n( "Sets the number of frames\nto run within the game" ) );
	textLabel6->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	layout23->addWidget( textLabel6 );

	QHBoxLayout* layout22_2;
	layout22_2 = new QHBoxLayout( 0, 0, 6, "layout22_2");
	QSpacerItem* spacer_3 = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout22_2->addItem( spacer_3 );

	spnFrames = new QSpinBox( page7, "spnFrames" );
	layout22_2->addWidget( spnFrames );
	layout23->addLayout( layout22_2 );
	layout28->addLayout( layout23 );

	QVBoxLayout* layout25;
	layout25 = new QVBoxLayout( 0, 0, 6, "layout25");

	QLabel* textLabel2;
	textLabel2 = new QLabel( page7, "textLabel2" );
	textLabel2->setText( i18n( "Set Y-Scale aspect ratio" ) );
	textLabel2->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	layout25->addWidget( textLabel2 );

	QHBoxLayout* layout24;
	layout24 = new QHBoxLayout( 0, 0, 6, "layout24");
	QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout24->addItem( spacer_4 );

	spnXScale = new QSpinBox( page7, "spnXScale" );
	layout24->addWidget( spnXScale );
	layout25->addLayout( layout24 );
	layout28->addLayout( layout25 );

	QVBoxLayout* layout27;
	layout27 = new QVBoxLayout( 0, 0, 6, "layout27");

	QLabel* textLabel3;
	textLabel3 = new QLabel( page7, "textLabel3" );
	textLabel3->setText( i18n( "Set X-Scale aspect ratio" ) );
	textLabel3->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	layout27->addWidget( textLabel3 );

	QHBoxLayout* layout26;
	layout26 = new QHBoxLayout( 0, 0, 6, "layout26");
	QSpacerItem* spacer_5 = new QSpacerItem( 41, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout26->addItem( spacer_5 );

	spnYScale = new QSpinBox( page7, "spnYScale" );
	layout26->addWidget( spnYScale );
	layout27->addLayout( layout26 );
	layout28->addLayout( layout27 );
	layout29->addLayout( layout28 );

	Form1Layout->addMultiCellLayout( layout29, 0, 0, 0, 1 );

	QVBoxLayout* layout52;
	layout52 = new QVBoxLayout( 0, 0, 6, "layout52");

	QHBoxLayout* layout50;
	layout50 = new QHBoxLayout( 0, 0, 6, "layout50");

	QLabel* textLabel1;
	textLabel1 = new QLabel( page7, "textLabel1" );
	textLabel1->setText( i18n( "Scale video to exactly\nthis height (0 = disable)" ) );
	layout50->addWidget( textLabel1 );

	spnScaleVideo = new QSpinBox( page7, "spnScaleVideo" );
	layout50->addWidget( spnScaleVideo );
	layout52->addLayout( layout50 );

	QVBoxLayout* layout51;
	layout51 = new QVBoxLayout( 0, 0, 6, "layout51");
	QSpacerItem* spacer_6 = new QSpacerItem( 20, 21, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layout51->addItem( spacer_6 );

	QLabel* textLabel4;
	textLabel4 = new QLabel( page7, "textLabel4" );
	textLabel4->setText( i18n( "Artwork resolution (0 for auto)" ) );
	textLabel4->setAlignment( int( QLabel::AlignVCenter ) );
	layout51->addWidget( textLabel4 );

	spnArtResolution = new QSpinBox( page7, "spnArtResolution" );
	layout51->addWidget( spnArtResolution );
	layout52->addLayout( layout51 );

	Form1Layout->addLayout( layout52, 1, 0 );

	QVBoxLayout* layout54;
	layout54 = new QVBoxLayout( 0, 0, 6, "layout54");

	QVBoxLayout* layout35;
	layout35 = new QVBoxLayout( 0, 0, 6, "layout35");

	QLabel* textLabel5;
	textLabel5 = new QLabel( page7, "textLabel5" );
	textLabel5->setText( i18n( "Set the gamma-correction (0.5-2.0)" ) );
	textLabel5->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	layout35->addWidget( textLabel5 );

	QHBoxLayout* layout34;
	layout34 = new QHBoxLayout( 0, 0, 6, "layout34");
	QSpacerItem* spacer_7 = new QSpacerItem( 144, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout34->addItem( spacer_7 );

	edtGamma = new QLineEdit( page7, "edtGamma" );
	layout34->addWidget( edtGamma );
	layout35->addLayout( layout34 );
	layout54->addLayout( layout35 );

	QLabel* textLabel1_2;
	textLabel1_2 = new QLabel( page7, "textLabel1_2" );
	textLabel1_2->setText( i18n( "Set X-Y Scale to the same aspect ratio" ) );
	textLabel1_2->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	layout54->addWidget( textLabel1_2 );

	QHBoxLayout* layout53;
	layout53 = new QHBoxLayout( 0, 0, 6, "layout53");
	QSpacerItem* spacer_8 = new QSpacerItem( 109, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout53->addItem( spacer_8 );

	edtAspectRatio = new QLineEdit( page7, "edtAspectRatio" );
	layout53->addWidget( edtAspectRatio );
	layout54->addLayout( layout53 );

	Form1Layout->addLayout( layout54, 1, 1 );

}

void XMameCfg::modScreen()
{
	QFrame *page2 = addPage( i18n("Screen"), i18n("Screen"), locate("data","kmameleon/pics/mod_screen.png") );
	QVBoxLayout *topLayout = new QVBoxLayout( page2, 0, 6 );

	chkX11FullScreen = new QCheckBox(i18n("Show full screen in X11"), page2);
	chkSdlFullScreen = new QCheckBox(i18n("Show full screen in SDL"), page2);
	chkGlFullScreen = new QCheckBox(i18n("Show full screen in GL"), page2);
	chkSvgaLibFullScreen = new QCheckBox(i18n("Show full screen in SVGALIB"), page2);

	topLayout->addWidget( chkSdlFullScreen,0,0 );
	topLayout->addWidget( chkX11FullScreen,0,0 );
	topLayout->addWidget( chkGlFullScreen,0,0 );
	topLayout->addWidget( chkSvgaLibFullScreen,0,0 );

	QSpacerItem* spacer = new QSpacerItem( 0, 210, QSizePolicy::Minimum, QSizePolicy::Expanding );
	topLayout->addItem( spacer );
}

void XMameCfg::modDevices()
{
	QFrame *page6 = addPage( i18n("Devices"), i18n("Devices"), locate("data","kmameleon/pics/mod_devices.png") );
	QVBoxLayout *topLayout = new QVBoxLayout( page6, 0, 6 );

	QLabel *lblJoyType = new QLabel(i18n("Select joystick type"),page6);
	QLabel *lblJoyDevice = new QLabel(i18n("Joystick device prefix:"), page6);

	inputBox = new QComboBox(page6);
	inputBox->insertItem(i18n("No joystick"));
	inputBox->insertItem(i18n("i386 style joystick driver (if compiled in)"));
	inputBox->insertItem(i18n("Fm Town Pad support (if compiled in)"));
	inputBox->insertItem(i18n("X11 input extension joystick (if compiled in)"));
	inputBox->insertItem(i18n("new i386 linux 1.x.x joystick driver(if compiled in)"));
	inputBox->insertItem(i18n("NetBSD USB joystick driver (if compiled in)"));

	chkAnalogStick = new QCheckBox(i18n("Use Joystick as analog for analog controls"), page6);
	customJoyDevName = new QLineEdit(page6, "customJojDevName");
	chkUseHotRod = new QCheckBox(i18n("Enable HotRod joystick support"), page6);
	chkUseHotRodSe = new QCheckBox(i18n("Select HotRod SE joystick support"), page6);
	chkUsbPsPad = new QCheckBox(i18n("The Joystick(s) are USB PS Game Pads"), page6);
	chkRapidFire = new QCheckBox(i18n("Enable rapid-fire support for joysticks"), page6);
	chkGrabMouse = new QCheckBox(i18n("Grab mouse (Good idea for xgl xmame)"), page6);

	topLayout->addWidget(lblJoyType, 0, 0);
	topLayout->addWidget(inputBox, 1, 0);

	topLayout->addWidget(chkAnalogStick, 2, 0);
	topLayout->addWidget(lblJoyDevice,3,0);
	topLayout->addWidget(customJoyDevName, 4, 0);
	topLayout->addWidget(chkUseHotRod, 5, 0);
	topLayout->addWidget(chkUseHotRodSe, 6, 0);
	topLayout->addWidget(chkUsbPsPad, 7, 0);
	topLayout->addWidget(chkRapidFire, 8, 0);
	topLayout->addWidget(chkGrabMouse, 9, 0);

	QSpacerItem* spacer = new QSpacerItem( 0, 210, QSizePolicy::Minimum, QSizePolicy::Expanding );
	topLayout->addItem( spacer );
}


void XMameCfg::modPaths()
{
	QFrame *page3 = addPage( i18n("Paths"), i18n("Paths"), locate("data","kmameleon/pics/mod_paths.png") );
	QVBoxLayout *topLayout = new QVBoxLayout( page3, 0, 0 );

	QHBoxLayout *layout1 = new QHBoxLayout (page3, 0, 0,"layout1");
	QHBoxLayout *layout2 = new QHBoxLayout (page3, 0, 0,"layout2");
	QHBoxLayout *layout3 = new QHBoxLayout (page3, 0, 0,"layout3");
	QHBoxLayout *layout4 = new QHBoxLayout (page3, 0, 0,"layout4");
	QHBoxLayout *layout5 = new QHBoxLayout (page3, 0, 0,"layout5");
	QHBoxLayout *layout6 = new QHBoxLayout (page3, 0, 0,"layout6");
	QHBoxLayout *layout7 = new QHBoxLayout (page3, 0, 0,"layout7");

//paths...
	QLabel *lblXmameRoms = new QLabel(i18n("XMame ROM path:"), page3);
	QLabel *lblXmameSamples = new QLabel(i18n("XMame samples path:"), page3);
	QLabel *lblXmameArt = new QLabel(i18n("XMame artwork path:"), page3);

	QLabel *lblXmameSdlBin = new QLabel(i18n("XMame SDL binary location:"), page3);
	QLabel *lblXmameX11Bin = new QLabel(i18n("XMame X11 binary location:"), page3);
	QLabel *lblXmameGlBin = new QLabel(i18n("XMame OpenGL binary location:"), page3);
	QLabel *lblXmameSvgaLibBin = new QLabel(i18n("XMame SvgaLib binary location:"), page3);

	pthXmameRoms = new QLineEdit(page3);
	pthXmameSamples = new QLineEdit(page3);
	pthXmameArt = new QLineEdit(page3);

	pthXmameSdlBin = new QLineEdit(page3);
	pthXmameX11Bin = new QLineEdit(page3);
	pthXmameGlBin = new QLineEdit(page3);
	pthXmameSvgaLibBin = new QLineEdit(page3);

//some attributes...
	pthXmameRoms->setReadOnly(true);
	pthXmameSamples->setReadOnly(true);
	pthXmameArt->setReadOnly(true);

	pthXmameSdlBin->setReadOnly(true);
	pthXmameX11Bin->setReadOnly(true);
	pthXmameGlBin->setReadOnly(true);
	pthXmameSvgaLibBin->setReadOnly(true);

	QToolButton *btnSelRoms = new QToolButton(page3);
	QToolButton *btnSelSamples = new QToolButton(page3);
	QToolButton *btnSelArt = new QToolButton(page3);
	btnSelRoms->setText("...");
	btnSelSamples->setText("...");
	btnSelArt->setText("...");

	QToolButton *btnSelSdlBin = new QToolButton(page3);
	QToolButton *btnSelX11Bin = new QToolButton(page3);
	QToolButton *btnSelGlBin = new QToolButton(page3);
	QToolButton *btnSelSvgaLibBin = new QToolButton(page3);
	QToolButton *btnRedetect = new QToolButton(page3);
	btnSelSdlBin->setText("...");
	btnSelX11Bin->setText("...");
	btnSelGlBin->setText("...");
	btnSelSvgaLibBin->setText("...");

//connections...
	connect(btnSelSdlBin, SIGNAL(clicked()), this, SLOT(slotSetSdlMameBin()) );
	connect(btnSelX11Bin, SIGNAL(clicked()), this, SLOT(slotSetX11MameBin()) );
	connect(btnSelGlBin, SIGNAL(clicked()), this, SLOT(slotSetGlMameBin()) );
	connect(btnSelSvgaLibBin, SIGNAL(clicked()), this, SLOT(slotSetSvgaLibMameBin()) );
	connect(btnSelRoms, SIGNAL(clicked()), this, SLOT(slotSetXMameRomDir()) );
	connect(btnSelSamples, SIGNAL(clicked()), this, SLOT(slotSetXMameSamplesDir()) );
	connect(btnSelArt, SIGNAL(clicked()), this, SLOT(slotSetXMameArtDir()) );
	connect(btnRedetect, SIGNAL(clicked()), this, SLOT(reload()) );

	layout1->addWidget(pthXmameRoms, 0, 0);
	layout1->addWidget(btnSelRoms, 0, 1);
	layout2->addWidget(pthXmameSdlBin, 0, 0);
	layout2->addWidget(btnSelSdlBin, 0, 1);
	layout3->addWidget(pthXmameX11Bin, 0, 0);
	layout3->addWidget(btnSelX11Bin, 0, 1);
	layout4->addWidget(pthXmameGlBin, 0, 0);
	layout4->addWidget(btnSelGlBin, 0, 1);
	layout5->addWidget(pthXmameSvgaLibBin, 0, 0);
	layout5->addWidget(btnSelSvgaLibBin, 0, 1);
	layout6->addWidget(pthXmameSamples, 0, 0);
	layout6->addWidget(btnSelSamples, 0, 1);
	layout7->addWidget(pthXmameArt, 0, 0);
	layout7->addWidget(btnSelArt, 0, 1);

	topLayout->addWidget( lblXmameRoms,0,0 );
	topLayout->addLayout(layout1, 1);
	topLayout->addWidget( lblXmameSamples,2,0 );
	topLayout->addLayout(layout6, 3);
	topLayout->addWidget( lblXmameArt,4,0 );
	topLayout->addLayout(layout7, 5);

	topLayout->addWidget( lblXmameSdlBin,6,0 );
	topLayout->addLayout(layout2, 7);
	topLayout->addWidget( lblXmameX11Bin,8,0 );
	topLayout->addLayout( layout3,9 );
	topLayout->addWidget( lblXmameGlBin,10,0 );
	topLayout->addLayout( layout4,11 );
	topLayout->addWidget( lblXmameSvgaLibBin,12,0 );
	topLayout->addLayout( layout5,13 );

	btnRedetect->setText(i18n("Redetect values"));
	topLayout->addWidget(btnRedetect);

	QSpacerItem* spacer = new QSpacerItem( 0, 410, QSizePolicy::Minimum, QSizePolicy::Expanding );
	topLayout->addItem( spacer );
}

void XMameCfg::slotSetSdlMameBin()
{
	QString binDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame SDL binary directoy"));
	if (! binDir.isEmpty())
	{
		QFileInfo fi(binDir+"xmame.SDL");
		if (fi.exists())
			pthXmameSdlBin->setText(binDir+"xmame.SDL");
		else
			KMessageBox::sorry(this,i18n("I can't locate XMame binaries at that location..."));
	}
}

void XMameCfg::slotSetX11MameBin()
{
	QString binDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame X11 binary directoy"));
	if (! binDir.isEmpty())
	{
		QFileInfo fi(binDir+"xmame.x11");
		if (fi.exists())
			pthXmameX11Bin->setText(binDir+"xmame.x11");
		else
			KMessageBox::sorry(this,i18n("I can't locate XMame binaries at that location..."));
	}
}

void XMameCfg::slotSetGlMameBin()
{
	QString binDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame XGL binary directoy"));
	if (! binDir.isEmpty())
	{
		QFileInfo fi(binDir+"xmame.xgl");
		if (fi.exists())
			pthXmameGlBin->setText(binDir+"xmame.xgl");
		else
			KMessageBox::sorry(this,i18n("I can't locate XMame binaries at that location..."));
	}
}

void XMameCfg::slotSetSvgaLibMameBin()
{
	QString binDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame X11 binary directoy"));
	if (! binDir.isEmpty())
	{
		QFileInfo fi(binDir+"xmame.svgalib");
		if (fi.exists())
			pthXmameSvgaLibBin->setText(binDir+"xmame.svgalib");
		else
			KMessageBox::sorry(this,i18n("I can't locate XMame binaries at that location..."));
	}
}

void XMameCfg::slotSetXMameRomDir()
{
	QString romDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame ROM directoy"));
	if (! romDir.isEmpty())
		pthXmameRoms->setText(romDir);
}

void XMameCfg::slotSetXMameSamplesDir()
{
	QString samplesDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame ROM directoy"));
	if (! samplesDir.isEmpty())
		pthXmameSamples->setText(samplesDir);
}

void XMameCfg::slotSetXMameArtDir()
{
	QString artDir = KFileDialog::getExistingDirectory(QString::null, this, i18n("Select XMame ROM directoy"));
	if (! artDir.isEmpty())
		pthXmameArt->setText(artDir);
}

void XMameCfg::modFirstRun()
{
	reload();

//first run...
	config->setGroup("First_run");
	config->writeEntry("First","false");

//Default XMame path...
	config->setGroup("general");
	config->writeEntry("xmame_sdl",pthXmameSdlBin->text());
	config->writeEntry("xmame_x11",pthXmameX11Bin->text());
	config->writeEntry("xmame_gl",pthXmameGlBin->text());
	config->writeEntry("xmame_svgalib",pthXmameSvgaLibBin->text());

	config->writeEntry("showOutput",true);

	config->setGroup("audio");
	config->writeEntry("issound",true);

	chkOutput->setChecked(true);
	chkIsSound->setChecked(true);

	config->sync();
}

void XMameCfg::reload() //needed, because firstRun shouldn't set the rest of the vars than path back...
{
	kdDebug() << "reload() loaded" << endl;
	QStringList xMameBinList;
	//this will be overwritten unconditinally

	pthXmameSdlBin->setText(KStandardDirs::findExe("xmame.SDL"));
	pthXmameX11Bin->setText(KStandardDirs::findExe("xmame.x11"));
	pthXmameGlBin->setText(KStandardDirs::findExe("xmame.xgl"));
	pthXmameSvgaLibBin->setText(KStandardDirs::findExe("xmame.svgalib"));

//could this be easier?
	if (KStandardDirs::findExe("xmame.SDL") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.SDL");
	if (KStandardDirs::findExe("xmame.x11") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.x11");
	if (KStandardDirs::findExe("xmame.xgl") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.xgl");
	if (KStandardDirs::findExe("xmame.svgalib") !="")
		xMameBinList +=KStandardDirs::findExe("xmame.svgalib");

	kdDebug() << "Executables count: " << xMameBinList.count() << endl;

	if (xMameBinList.count() == 1)
	{
		kdDebug() << xMameBinList[0] << endl;
		QString options=xMameBinList[0];
		options +=" -sc ";
		options +=" | grep -i / ";

		adProc.clearArguments();
		#if KDE_VERSION >= 306
			adProc.setUseShell(true);
		#endif

		adProc << options.local8Bit();
		if (adProc.start(KProcess::NotifyOnExit, KProcess::AllOutput) == false)
			kdDebug() << "something wrong" << endl;
		else
			kdDebug() << "Process running" << endl;

	}
	else if ( xMameBinList.count() > 1)
		KMessageBox::information(this, i18n("There are more than one xmame binary detected... \nPlease use pathfinder tool to detect default paths"));
	else if ( xMameBinList.count() < 1)
		KMessageBox::information(this, i18n("Cannot find XMame binaries... Make sure that it is installed..."));
}

void XMameCfg::loadConfig()
{
	config->setGroup("general");
	pthXmameRoms->setText(config->readEntry("xmame_rom",""));
	pthXmameSamples->setText(config->readEntry("xmame_samples",""));
	pthXmameArt->setText(config->readEntry("xmame_art",""));

	pthXmameSdlBin->setText(config->readEntry("xmame_sdl",""));
	pthXmameX11Bin->setText(config->readEntry("xmame_x11",""));
	pthXmameGlBin->setText(config->readEntry("xmame_gl",""));
	pthXmameSvgaLibBin->setText(config->readEntry("xmame_svga",""));

	chkOutput->setChecked(config->readBoolEntry("showOutput",true));
	chkColorOutput->setChecked(config->readBoolEntry("color_debug",false));
	cbConfigType->setCurrentItem(config->readNumEntry("setmode", 0));
	chkCheatMode->setChecked(config->readBoolEntry("cheatMode", false));
	chkDetachProcess->setChecked(config->readBoolEntry("detachProcess", false));

	xMameType->setCurrentItem(config->readNumEntry("xmame_type",0));

	config->setGroup("screen");
	chkSdlFullScreen->setChecked(config->readBoolEntry("sdl_fullscreen",false));
	chkX11FullScreen->setChecked(config->readBoolEntry("x11_fullscreen",false));
	chkGlFullScreen->setChecked(config->readBoolEntry("gl_fullscreen",false));
	chkSvgaLibFullScreen->setChecked(config->readBoolEntry("svga_fullscreen",false));

	config->setGroup("audio");
	chkIsSound->setChecked(config->readBoolEntry("issound", true));
	chkSamples->setChecked(config->readBoolEntry("samples", false));
	chkFakeSound->setChecked(config->readBoolEntry("fakesound", false));
	audioSetVolume->setValue(config->readNumEntry("volume",-(16)));

	chkCustomAudioSf->setChecked(config->readBoolEntry("customsamplefrequency",false));
	chkCustomAudioBufSize->setChecked(config->readBoolEntry("custombufsize",false));
	chkCustomAudioDevice->setChecked(config->readBoolEntry("customaudiodevice",false));
	chkCustomAudioMixerDevice->setChecked(config->readBoolEntry("custommixerdevice",false));

	audioSfEdit->setText(config->readEntry("samplefrequency",""));
	audioBufSizeEdit->setText(config->readEntry("bufsize",""));
	audioDeviceEdit->setText(config->readEntry("audiodevice",""));
	audioMixerDeviceEdit->setText(config->readEntry("mixerdevice",""));

	config->setGroup("video");
	videoFx->setCurrentItem(config->readNumEntry("effect",0));
	cbRotation->setCurrentItem(config->readNumEntry("rotation",0));
	spnMaxAutoFrameSkip->setValue(config->readNumEntry("maxautoframeskip",0));
	spnNoAutoFrameSkip->setValue(config->readNumEntry("noautoframeskip",0));
	chkEnableFrameSkip->setChecked(config->readBoolEntry("enableframeskip",false));
	chkFrameSkip->setChecked(config->readBoolEntry("enableautoframeskip",false));
	chkFlipX->setChecked(config->readBoolEntry("flipx",false));
	chkFlipY->setChecked(config->readBoolEntry("flipy",false));

	config->setGroup("effects");
	chkScanlines->setChecked(config->readBoolEntry("enablescanlines",false));
	chkArtwork->setChecked(config->readBoolEntry("enableartwork",false));
	chkOverlay->setChecked(config->readBoolEntry("enableoverlays",false));
	chkBezel->setChecked(config->readBoolEntry("enablebezel",false));
	chkArtCrop->setChecked(config->readBoolEntry("enableartcrop",false));
	chkThrottle->setChecked(config->readBoolEntry("enablethrottle",false));
	chkSleepIdle->setChecked(config->readBoolEntry("enablesleepidle",false));

	spnBrightness->setValue(config->readNumEntry("brighteness",0));
	spnFrames->setValue(config->readNumEntry("fframes",0));
	spnXScale->setValue(config->readNumEntry("xscale",0));
	spnYScale->setValue(config->readNumEntry("yscale",0));
	spnScaleVideo->setValue(config->readNumEntry("scalevideo",0));
	spnArtResolution->setValue(config->readNumEntry("artres",0));

	config->writeEntry("gamma",edtGamma->text());
	config->writeEntry("aspectratio",edtAspectRatio->text());

	config->setGroup("devices");
	inputBox->setCurrentItem(config->readNumEntry("JoystickType",0));
	chkAnalogStick->setChecked(config->readBoolEntry("AnalogStick", false));
	customJoyDevName->setText(config->readEntry("JoyDevice", "/dev/js"));
	chkUseHotRod->setChecked(config->readBoolEntry("UseHotRod", false));
	chkUseHotRodSe->setChecked(config->readBoolEntry("UseHotRodSe", false));
	chkUsbPsPad->setChecked(config->readBoolEntry("UseUsbPsPad", false));
	chkRapidFire->setChecked(config->readBoolEntry("EnableRapidFire", false));
	chkGrabMouse->setChecked(config->readBoolEntry("GrabMouse", false));
}

void XMameCfg::slotOk()
{
	writeConfig();
	emit listenConfOk();
	close();
}

void XMameCfg::writeConfig()
{
	config->setGroup("general");
	config->writeEntry("xmame_rom",pthXmameRoms->text());
	config->writeEntry("xmame_samples",pthXmameSamples->text());
	config->writeEntry("xmame_art",pthXmameArt->text());

	config->writeEntry("xmame_sdl",pthXmameSdlBin->text());
	config->writeEntry("xmame_x11",pthXmameX11Bin->text());
	config->writeEntry("xmame_gl",pthXmameGlBin->text());
	config->writeEntry("xmame_svga",pthXmameSvgaLibBin->text());

	config->writeEntry("showOutput",chkOutput->isChecked());
	config->writeEntry("color_debug",chkColorOutput->isChecked());
	config->writeEntry("xmame_type",xMameType->currentItem());
	config->writeEntry("setmode", cbConfigType->currentItem());
	config->writeEntry("cheatMode", chkCheatMode->isChecked());
	config->writeEntry("detachProcess", chkDetachProcess->isChecked());


	config->setGroup("screen");
	config->writeEntry("sdl_fullscreen",chkSdlFullScreen->isChecked());
	config->writeEntry("x11_fullscreen",chkX11FullScreen->isChecked());
	config->writeEntry("gl_fullscreen",chkGlFullScreen->isChecked());
	config->writeEntry("svga_fullscreen",chkSvgaLibFullScreen->isChecked());

	config->setGroup("audio");
	config->writeEntry("issound",chkIsSound->isChecked());
	config->writeEntry("samples",chkSamples->isChecked());
	config->writeEntry("fakesound",chkFakeSound->isChecked());
	config->writeEntry("volume",audioSetVolume->value());

	config->writeEntry("customsamplefrequency",chkCustomAudioSf->isChecked());
	config->writeEntry("custombufsize",chkCustomAudioBufSize->isChecked());
	config->writeEntry("customaudiodevice",chkCustomAudioDevice->isChecked());
	config->writeEntry("custommixerdevice",chkCustomAudioMixerDevice->isChecked());

	config->writeEntry("samplefrequency",audioSfEdit->text());
	config->writeEntry("bufsize",audioBufSizeEdit->text());
	config->writeEntry("audiodevice",audioDeviceEdit->text());
	config->writeEntry("mixerdevice",audioMixerDeviceEdit->text());

	config->setGroup("video");
	config->writeEntry("effect",videoFx->currentItem());
	config->writeEntry("maxautoframeskip",spnMaxAutoFrameSkip->value());
	config->writeEntry("noautoframeskip",spnNoAutoFrameSkip->value());
	config->writeEntry("enableframeskip",chkEnableFrameSkip->isChecked());
	config->writeEntry("enableautoframeskip",chkFrameSkip->isChecked());
	config->writeEntry("rotation",cbRotation->currentItem());
	config->writeEntry("flipx",chkFlipX->isChecked());
	config->writeEntry("flipy",chkFlipY->isChecked());

	config->setGroup("effects");
	config->writeEntry("enablescanlines",chkScanlines->isChecked());
	config->writeEntry("enableartwork",chkArtwork->isChecked());
	config->writeEntry("enableoverlays",chkOverlay->isChecked());
	config->writeEntry("enablebezel",chkBezel->isChecked());
	config->writeEntry("enableartcrop",chkArtCrop->isChecked());
	config->writeEntry("enablethrottle",chkThrottle->isChecked());
	config->writeEntry("enablesleepidle",chkSleepIdle->isChecked());
	config->writeEntry("brighteness",spnBrightness->value());
	config->writeEntry("fframes",spnFrames->value());
	config->writeEntry("xscale",spnXScale->value());
	config->writeEntry("yscale",spnYScale->value());
	config->writeEntry("scalevideo",spnScaleVideo->value());
	config->writeEntry("artres",spnArtResolution->value());
	config->writeEntry("gamma",edtGamma->text());
	config->writeEntry("aspectratio",edtAspectRatio->text());

	config->setGroup("devices");
	config->writeEntry("JoystickType",inputBox->currentItem());
	config->writeEntry("AnalogStick", chkAnalogStick->isChecked());
	config->writeEntry("JoyDevice", customJoyDevName->text());
	config->writeEntry("UseHotRod", chkUseHotRod->isChecked());
	config->writeEntry("UseHotRodSe", chkUseHotRodSe->isChecked());
	config->writeEntry("UseUsbPsPad", chkUsbPsPad->isChecked());
	config->writeEntry("EnableRapidFire", chkRapidFire->isChecked());
	config->writeEntry("GrabMouse", chkGrabMouse->isChecked());
	config->sync();

//saving the current options...
	saveOptions();
}

void XMameCfg::saveOptions()
{
	QString temp;
//cheat options...
	if(chkCheatMode->isChecked())
		loadedOptions += " -cheat ";

//video
	loadedOptions += " -effect " + temp.setNum(videoFx->currentItem()) + " ";

	switch (cbRotation->currentItem())
	{
		case 1:
			loadedOptions += " -norotate ";
		break;
		case 2:
			loadedOptions += " -rol ";
		break;
		case 3:
			loadedOptions += " -ror ";
		break;
	}

if (chkEnableFrameSkip->isChecked())
{
	if (chkFrameSkip->isChecked())
		loadedOptions += " -autoframeskip ";
	else
		loadedOptions += " -noautoframeskip ";

	loadedOptions += " -maxautoframeskip " + temp.setNum(spnMaxAutoFrameSkip->value());
	loadedOptions += " -frameskip " + temp.setNum(spnNoAutoFrameSkip->value());
}
	if (chkFlipX->isChecked())
		loadedOptions += " -flipx ";

	if (chkFlipY->isChecked())
		loadedOptions += " -flipy ";

//screen
	if (xMameType->currentItem() ==1 && chkSdlFullScreen->isChecked() == true)
		loadedOptions +=" -fullscreen ";
	else if (xMameType->currentItem() ==2  && chkX11FullScreen->isChecked() == true)
		loadedOptions +=" -x11 1 ";
	else if (xMameType->currentItem() ==3  && chkGlFullScreen->isChecked() == true)
		loadedOptions +=" -fullscreen ";

//ROM path...
	loadedOptions += " -rompath " + pthXmameRoms->text();

//audio
	if (chkIsSound->isChecked() == false)
		loadedOptions += " -nosound ";
	if (chkSamples->isChecked() == false)
		loadedOptions += " -nosamples ";
	if (chkFakeSound->isChecked() == true)
		loadedOptions += " -fakesound ";

	loadedOptions += " -volume "+QString::number(audioSetVolume->value())+" ";

	if(chkCustomAudioSf->isChecked() && audioSfEdit->text() !="")
		loadedOptions += " -samplefreq " + audioSfEdit->text();
	if(chkCustomAudioBufSize->isChecked() && audioBufSizeEdit->text() !="")
		loadedOptions += " -bufsize " +audioBufSizeEdit->text();
	if(chkCustomAudioDevice->isChecked() && audioDeviceEdit->text() !="")
		loadedOptions += " -audiodevice "+audioDeviceEdit->text();
	if(chkCustomAudioMixerDevice->isChecked() && audioMixerDeviceEdit->text() !="")
		loadedOptions += " -mixerdevice " +audioMixerDeviceEdit->text();

//input devices
	loadedOptions += " -joytype " + QString::number(inputBox->currentItem());
	if (customJoyDevName->text() !="")
		loadedOptions += " -jdev " + customJoyDevName->text();

	if (chkAnalogStick->isChecked() == true)
		loadedOptions += " -analogstick ";
	if (chkUseHotRod->isChecked() == true)
		loadedOptions += " -hotrod ";
	if (chkUseHotRodSe->isChecked() == true)
		loadedOptions += " -hotrodse ";
	if (chkUsbPsPad->isChecked() == true)
		loadedOptions += " -usbpspad ";
	if (chkRapidFire->isChecked() == true)
		loadedOptions += " -rapidfire ";
	if (chkGrabMouse->isChecked() == true)
		loadedOptions += " -grabmouse ";


//these are advanced options -- disabled in simple mode
	if (cbConfigType->currentItem() == 1)
	{
		if (chkScanlines->isChecked() == true)
			loadedOptions += " -scanlines ";
		else
			loadedOptions += " -noscanlines ";

		if (chkArtwork->isChecked() == true)
			loadedOptions += " -artwork ";
		else
			loadedOptions += " -noartwork ";

		if (chkBackdrops->isChecked() == true)
			loadedOptions += " -backdrop ";
		else
			loadedOptions += " -nobackdrop ";

		if (chkOverlay->isChecked() == true)
			loadedOptions += " -overlay ";
		else
			loadedOptions += " -nooverlay ";

		if (chkBezel->isChecked() == true)
			loadedOptions += " -bezel ";
		else
			loadedOptions += " -nobezel ";

		if (chkArtCrop->isChecked() == true)
			loadedOptions += " -artcrop ";
		else
			loadedOptions += " -noartcrop ";

		if (chkThrottle->isChecked() == true)
			loadedOptions += " -th ";
		else
			loadedOptions += " -noth ";

		if (chkSleepIdle->isChecked() == true)
			loadedOptions += " -si ";
		else
			loadedOptions += " -nosi ";

		if (spnBrightness->value() > 0)
			loadedOptions += " -brt " + QString::number(spnBrightness->value());

		if (spnFrames->value() > 0)
			loadedOptions += " -ftr " + QString::number(spnFrames->value());

		if (spnXScale->value() > 0)
			loadedOptions += " -hs " + QString::number(spnXScale->value());

		if (spnYScale->value() > 0)
			loadedOptions += " -ws " + QString::number(spnXScale->value());

		if (spnScaleVideo->value() > 0)
			loadedOptions += " -ah " + QString::number(spnScaleVideo->value());

		if (spnArtResolution->value() > 0)
			loadedOptions += " -artres " + QString::number(spnArtResolution->value());

		if (edtGamma->text() !="")
			loadedOptions += " -gc " + edtGamma->text();

		if (edtAspectRatio->text() !="")
			loadedOptions += " -scale " + edtAspectRatio->text();

	}

//writing current options...
	config->setGroup("xmame_options");
	config->writeEntry("current_options", loadedOptions);

//ending.. It's a quick & dirty fix; Sorry...
	loadedOptions="";
}

void XMameCfg::setDefaultPaths(KProcess *, char *text, int len)
{
	kdDebug() << "setDefaultPaths() running" << endl;
	QStringList list;

	QString temp;
	int pos;
	QString dataStream;

	temp.fill(' ',len+1);
	temp.replace(0,len,text);
	temp[len]='\0';
	paths=QStringList::split("\n", temp.local8Bit());

	for ( unsigned int i= 0; i<paths.count(); i++)
	{
		if (paths[i].contains("rom") > 0 && paths[i].contains("/") > 0)
		{
			kdDebug() << paths[i] << endl;
			list += paths[i];
		}
		if (paths[i].contains("samples") > 0 && paths[i].contains("/") > 0)
		{
			kdDebug() << paths[i] << endl;
			list += paths[i];
		}
		if (paths[i].contains("artwork") > 0 && paths[i].contains("/") > 0)
		{
			kdDebug() << paths[i] << endl;
			list += paths[i];
		}
	}

	for ( unsigned int i= 0; i<list.count(); i++)
		kdDebug() << list[i] << endl;

//this will handle the paths...
	config->setGroup("general");
	for (int pthEnv =0; pthEnv <=2; pthEnv++)
	{
		switch(pthEnv)
		{
			case 0: //rompath
				pos = list[pthEnv].find("/");
				dataStream = list[pthEnv].mid(pos);
				kdDebug() << dataStream << endl;
				if (config->readEntry("xmame_rom","").isEmpty())
				{
					pthXmameRoms->setText(dataStream);
					kdDebug() << dataStream << endl;
					config->writeEntry("xmame_rom",dataStream);
				}
				else
					if (KMessageBox::questionYesNo(this,i18n("This will override Your current rom path settings... Want me to proceed?")) == 3)
					{
						pthXmameRoms->setText(dataStream);
						kdDebug() << dataStream << endl;
					}
			break;
			case 1: //samplepath
				pos = list[pthEnv].find("/");
				dataStream = list[pthEnv].mid(pos);
				kdDebug() << dataStream << endl;
				if (config->readEntry("xmame_samples","").isEmpty())
				{
					pthXmameSamples->setText(dataStream);
					kdDebug() << dataStream << endl;
				}
				else
					if (KMessageBox::questionYesNo(this,i18n("This will override Your current sample path settings... Want me to proceed?")) == 3)
					{
						pthXmameSamples->setText(dataStream);
						kdDebug() << dataStream << endl;
					}
			break;
			case 2: //artworkpath | this can be wrong in xmame > 0.64
				pos = list[pthEnv].find("/");
				dataStream = list[pthEnv].mid(pos);
				kdDebug() << dataStream << endl;
				if (config->readEntry("xmame_art","").isEmpty())
				{
					pthXmameArt->setText(dataStream);
					kdDebug() << dataStream << endl;
				}
				else
					if (KMessageBox::questionYesNo(this,i18n("This will override Your current artwork path settings... Want me to proceed?")) == 3)
					{
						pthXmameArt->setText(dataStream);
						kdDebug() << dataStream << endl;
					}
			break;
		} //end switch
	} // end for loop
	config->sync();
	emit listenConfOk();
//a warning...
	if (pthXmameRoms->text() =="")
		KMessageBox::error(this, i18n("KMamelon can't find the path to XMame roms... \nPlease configure it..."));

	list = 0L;
}

void XMameCfg::changeVolumeLevel(int lvlChanged)
{
	lblDisplayVolume->setText(QString::number(lvlChanged) +i18n(" dB"));
}

void XMameCfg::slotConfigType(int cfgLevel)
{
	kdDebug() << cfgLevel << endl;
}

void XMameCfg::setVersionInfo(KProcess *, char *text, int len)
{
	QString temp;

	temp.fill(' ',len+1);
	temp.replace(0,len,text);
	temp[len]='\0';
	kdDebug() << temp << endl;

	versionBrowser -> insert(temp.local8Bit());
}

#include "xmameconfig.moc"

